# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy import and_

from basesoft.web.contrib.license.models import Group, Domain, Permission
from basesoft.web.contrib.i18n.models import Country, Locale
from basesoft.web.contrib.account.models import Account

from iqportal.tnt.clm.models import TntCLMWaveType, TntCLMWaveMainLabel
from sqlalchemy.exc import IntegrityError
from iqportal.tnt.response.models import TntCLMWaveRespondentType

def sync(session):

    # sync other stuff

    tnt_domain = None
    try:
        tnt_domain = session.query(Domain).filter(Domain.name=='TNT').one()
    except NoResultFound:
        print 'TNT clm::db_sync: domain TNT not found, skipping...'
        return
    group = tnt_domain.default_group

    # add some default types for clm.
    types = [{
        'label': "Wave 01-32",
        'module': 'clm_wave_01'
    },{
        'label': "Wave 33-35",
        'module': 'clm_wave_33'
    }, {
        'label': "Wave 36+",
        'module': 'clm_wave_36'
    }]

    current_types = [t.module for t in session.query(TntCLMWaveType).all()]
    for type in types:
        # if we already have a type for the same module, skip it for now.
        if type['module'] in current_types:
            continue
        t = TntCLMWaveType()
        for attr in type:
            setattr(t, attr, type[attr])

        object = t.get_group_object()[0]
        for privilege in t.all_privileges:
            go = object()
            go.group = group
            go.privilege = privilege
            go.group_has_privilege = True
            go.license_has_privilege = True
            session.add(go)
            t.groups.append(go)

        session.add(t)

    # wave main labels
    for id,label in [(0, 'Other Waves'), (1, 'Main Waves')]:
        l = TntCLMWaveMainLabel()
        l.id = id
        l.label = label
        session.add(l)
        try:
            session.flush()
        except IntegrityError:
            pass

    # respondent types
    for id,label in [(0, 'Non Respondent'), (1, 'Internet'), (2, 'Paper')]:
        l = TntCLMWaveRespondentType()
        l.id = id
        l.label = label
        session.add(l)
        try:
            session.flush()
        except IntegrityError:
            pass
