# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon

class WaveManagerIcon(IconBarIcon):
    """
    Class responsible for handling the wave manager icon
    """

    id = 'icon-wave-manager'
    title = _('Wave manager')
    icon_index = 75
    component = 'tnt_clm_wave_manager'
    group = 'tnt-clm'
    colorname = 'iconbar-blaze-orange'

    char_main = u'\uE836' # Clock icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        """
        controller.assert_permissions('tnt_clm_wave_manager')

class TntCLMAttributeManagerIcon(IconBarIcon):
    """
    Class responsible for handling the invoice icon
    """

    id = 'icon-tnt-clm-attribute-manager'
    title = _('TNT CLM Attributes')
    icon_index = 76
    component = 'tnt_clm_attribute_manager'
    group = 'tnt-clm'
    colorname = 'iconbar-blaze-orange'

    char_main = u'\uE837' # Closed box icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        """
        controller.assert_permissions('tnt_clm_attribute_manager')

class ScaleAnswerManagerIcon(IconBarIcon):
    """
    Class responsible for handling the scale answer manager icon
    """

    id = 'icon-scale-answer-manager'
    title = _('Scale answer manager')
    icon_index = 77
    component = 'tnt_clm_scale_answer_manager'
    group = 'tnt-clm'
    colorname = 'iconbar-blaze-orange'

    char_main = u'\uE838' # Message clouds icon
    char_sub = u'\uE80C' # Question mark icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('view_tnt_clm_scale_answer')

class LocaleManagerIcon(IconBarIcon):
    id = 'icon-clm-locale-manager'
    title = _('Locale manager')
    component = 'tnt_clm_locale_manager'
    icon_index = 78
    group = 'tnt-clm'
    colorname = 'iconbar-blaze-orange'

    char_main = u'\uE803' # 3 users icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        """
        controller.assert_permissions('tnt_clm_locale_manager')

class CLMMailManagerIcon(IconBarIcon):
    """
    Class responsible for handling the tnt wave mail manager icon
    """

    id = 'icon-clm-mail-manager'
    title = _('CLM Wave Mail manager')
    href = '#'
    icon_index = 80
    component = 'tnt_clm_mail_manager'
    group = 'tnt-clm'
    colorname = 'iconbar-blaze-orange'

    char_main = u'\uE81A' # Mail icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('view_mail', 'view_tnt_wave_mail')


def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """

    return [WaveManagerIcon(), TntCLMAttributeManagerIcon(), ScaleAnswerManagerIcon(), LocaleManagerIcon(), CLMMailManagerIcon(),]
