# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.
from iqportal.tnt.clm.models import TntCLMAttribute

import datetime
from sqlalchemy.orm import mapper, relation
from sqlalchemy import Column, Integer, ForeignKey, DateTime, UnicodeText, Unicode, \
    SmallInteger, Enum, UniqueConstraint, String, BigInteger

from basesoft.sqlalchemy.table import IQTable

from basesoft.web.contrib.license import models as license_models
from basesoft.web.contrib.account import models as account_models

from django.utils.translation import ugettext as _

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    """
    """
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        tblTntCLMAccount = IQTable('tblTntCLMAccount', metadata,
            Column('account_id', Integer, ForeignKey('tblAccount.id', ondelete='CASCADE'), primary_key=True)
        )
        tblTntCLMAccountLocale = IQTable('tblTntCLMAccountLocale', metadata,
            Column('account_id', Integer, ForeignKey('tblTntCLMAccount.account_id', ondelete='CASCADE'), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblTntCLMLocale.id'), primary_key=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tblTntCLMCustomer = IQTable('tblTntCLMCustomer', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),

            Column('account_number', BigInteger, nullable=False),
            Column('country_id', Integer, ForeignKey('tblCountry.id'), nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),

            UniqueConstraint('account_number', 'country_id', name='acc_country_index')
        )

        tblTntCLMContact = IQTable('tblTntCLMContact', metadata,
            Column('contact_id', Integer, ForeignKey('tblContact.id', ondelete='CASCADE'), primary_key=True)
        )

        tblTntCLMCompany = IQTable('tblTntCLMCompany', metadata,
            Column('company_id', Integer, ForeignKey('tblCompany.id', ondelete='CASCADE'), primary_key=True),
            Column('depot_id', Integer, ForeignKey('tblTntDepot.id')),
            Column('locale_id', Integer, ForeignKey('tblTntCLMLocale.id'), nullable=False),
            Column('customer_id', Integer, ForeignKey('tblTntCLMCustomer.id'), nullable=False),
            Column('revenue_band', Unicode(5)),
            Column('subsector', Integer),
            Column('imp_code', Integer),
            UniqueConstraint('depot_id', 'locale_id', 'customer_id', 'revenue_band', name='acc_country_index')
        )

        tblTntCLMService = IQTable('tblTntCLMService', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('label', Unicode(255), nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tblTntCLMLocale = IQTable('tblTntCLMLocale', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),

            Column('label', Unicode(5), nullable=False),
            Column('description', Unicode(128), nullable=False),
            Column('locale_id', Integer, ForeignKey('tblLocale.id'), nullable=True),
            Column('country_id', Integer, ForeignKey('tblCountry.id'), nullable=False),
            Column('service_id', Integer, ForeignKey('tblTntCLMService.id'), nullable=False),

            Column('language_code', String(2), nullable=True),
            Column('direction', Enum('ltr', 'rtl'), nullable=False, default='ltr'),
            Column('location', Enum('D', 'I'), nullable=True),
            Column('location_2015', Enum('D', 'I'), nullable=False),

            Column('active', SmallInteger, nullable=False, default=1),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        # the content and subject definitions in different locales.
        tblTntCLMWaveMailLocale = IQTable('tblTntCLMWaveMailLocale', metadata,
            Column('mail_id', Integer, ForeignKey('tblMail.id', ondelete='CASCADE'), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblTntCLMLocale.id'), primary_key=True),
            Column('subject', String(128), nullable=False),
            Column('content', UnicodeText, nullable=False),
            Column('link_label', UnicodeText, nullable=True),
            Column('status', SmallInteger, nullable=False, default=0),
            Column('rejection_reason', UnicodeText),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntCLMWaveMail = IQTable('tblTntCLMWaveMail', metadata,
            Column('mail_id', Integer, ForeignKey('tblMail.id', ondelete='CASCADE'), primary_key=True),
            Column('fallback_locale_id', Integer, ForeignKey('tblTntCLMLocale.id'), nullable=False),
        )

        tblTntCLMWaveMainLabel = IQTable('tblTntCLMWaveMainLabel', metadata,
            Column('id', SmallInteger, primary_key=True, autoincrement=False),
            Column('label', Unicode(255), nullable=False),
        )

        tblTntCLMWaveType = IQTable('tblTntCLMWaveType', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),

            Column('label', Unicode(255), nullable=False),
            Column('module', Unicode(255), nullable=False),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntCLMWaveTypeGroup = IQTable('tblTntCLMWaveTypeGroup', metadata,
            Column('type_id', Integer, ForeignKey('tblTntCLMWaveType.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntCLMScaleAnswer = IQTable('tblTntCLMScaleAnswer', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('default_label', UnicodeText, nullable=False),

            # for which type of Wave was this used.
            Column('type_id', Integer, ForeignKey('tblTntCLMWaveType.id', ondelete="RESTRICT")),
            Column('position', Integer, nullable=False),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntCLMScaleAnswerGroup = IQTable('tblTntCLMScaleAnswerGroup', metadata,
            Column('scale_answer_id', Integer, ForeignKey('tblTntCLMScaleAnswer.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntCLMScaleAnswerLocale = IQTable('tblTntCLMScaleAnswerLocale', metadata,
            Column('scale_answer_id', Integer, ForeignKey('tblTntCLMScaleAnswer.id', ondelete="CASCADE"), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblTntCLMLocale.id'), primary_key=True),

            Column('label', UnicodeText, nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntCLMAttribute = IQTable('tblTntCLMAttribute', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            # type of attributes, see the model for the defines.
            Column('type', SmallInteger, nullable=False, default=TntCLMAttribute.TYPE_SCALE),
            Column('default_label', UnicodeText, nullable=False),

            Column('priority_trigger_at', SmallInteger, nullable=True),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntCLMAttributeGroup = IQTable('tblTntCLMAttributeGroup', metadata,
            Column('attribute_id', Integer, ForeignKey('tblTntCLMAttribute.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # default translation and the properties of the attribute
        tblTntCLMAttributeLocale = IQTable('tblTntCLMAttributeLocale', metadata,
            Column('attribute_id', Integer, ForeignKey('tblTntCLMAttribute.id', ondelete="CASCADE"), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblTntCLMLocale.id'), primary_key=True),

            Column('label', UnicodeText, nullable=False),
            # priority reporting is triggered on this attribute for this locale-identifier
            # when response is lower than given integer
            Column('priority_trigger_at', SmallInteger, nullable=True),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # used for non TYPE_SCALE attributes.
        tblTntCLMAttributeAnswer = IQTable('tblTntCLMAttributeAnswer', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('attribute_id', Integer, ForeignKey('tblTntCLMAttribute.id', ondelete="CASCADE")),

            Column('default_label', UnicodeText, nullable=False),
            Column('position', Integer, nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # default translation of the attribute answer in case not TYPE_SCALE
        tblTntCLMAttributeAnswerLocale = IQTable('tblTntCLMAttributeAnswerLocale', metadata,
            Column('answer_id', Integer, ForeignKey('tblTntCLMAttributeAnswer.id', ondelete="CASCADE"), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblTntCLMLocale.id'), primary_key=True),

            Column('label', UnicodeText, nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntCLMWave = IQTable('tblTntCLMWave', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            # type of survey, see the model for the defines, this determines the layout of the survey.
            Column('type_id', Integer, ForeignKey('tblTntCLMWaveType.id', ondelete="RESTRICT")),
            Column('name', Unicode(255), nullable=False),
            Column('code', Unicode(255), nullable=False, default=u''),
            Column('is_main', SmallInteger, ForeignKey('tblTntCLMWaveMainLabel.id', ondelete='RESTRICT'), default=0),

            Column('status', SmallInteger, nullable=False, default=0),
            Column('begin_date', DateTime()),
            Column('end_date', DateTime(), nullable=True),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntCLMWaveGroup = IQTable('tblTntCLMWaveGroup', metadata,
            Column('wave_id', Integer, ForeignKey('tblTntCLMWave.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntCLMWaveLocaleRevision = IQTable('tblTntCLMWaveLocaleRevision', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('wave_id', Integer, ForeignKey('tblTntCLMWave.id', ondelete="CASCADE")),
            Column('locale_id', Integer, ForeignKey('tblTntCLMLocale.id')),
            Column('account_id', Integer, ForeignKey('tblAccount.id')),
            Column('status', SmallInteger, nullable=False, default=0),
            Column('rejection_reason', UnicodeText),

            Column('state', UnicodeText, nullable=True),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntCLMWaveLocale = IQTable('tblTntCLMWaveLocale', metadata,
            Column('locale_revision_id', Integer, ForeignKey('tblTntCLMWaveLocaleRevision.id'), primary_key=True),

            Column('header_text', UnicodeText, nullable=True),
            Column('sub_header_text', UnicodeText, nullable=True),

            Column('opt_out_text', UnicodeText, nullable=True),
            Column('closure_text', UnicodeText, nullable=True),

            Column('pdf_fontsize', Integer, nullable=True),

            Column('is_default', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # keep a reference to the original attribute for reporting purposes.
        tblTntCLMWaveAttribute = IQTable('tblTntCLMWaveAttribute', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('wave_id', Integer, ForeignKey('tblTntCLMWave.id', ondelete="CASCADE")),
            Column('attribute_id', Integer, ForeignKey('tblTntCLMAttribute.id', ondelete="CASCADE")),
            Column('category_id', Integer, ForeignKey('tblTntCLMWaveAttributeCategory.id', ondelete="RESTRICT"), nullable=True),

            Column('priority_trigger_at', SmallInteger, nullable=True),

            Column('position', Integer, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
            UniqueConstraint('wave_id', 'attribute_id')
        )

        tblTntCLMWaveAttributeCategory = IQTable('tblTntCLMWaveAttributeCategory', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('wave_id', Integer, ForeignKey('tblTntCLMWave.id', ondelete="CASCADE")),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntCLMWaveAttributeCategoryLocale = IQTable('tblTntCLMWaveAttributeCategoryLocale', metadata,
            Column('category_id', Integer, ForeignKey('tblTntCLMWaveAttributeCategory.id', ondelete="CASCADE"), primary_key=True),
            Column('locale_revision_id', Integer, ForeignKey('tblTntCLMWaveLocaleRevision.id'), primary_key=True),

            Column('label', UnicodeText, nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntCLMWaveScaleAnswer = IQTable('tblTntCLMWaveScaleAnswer', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('wave_id', Integer, ForeignKey('tblTntCLMWave.id', ondelete="CASCADE")),
            Column('scale_answer_id', Integer, ForeignKey('tblTntCLMScaleAnswer.id', ondelete="CASCADE")),

            Column('position', Integer, nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
            UniqueConstraint('wave_id', 'scale_answer_id')
        )

        tblTntCLMWaveScaleAnswerLocale = IQTable('tblTntCLMWaveScaleAnswerLocale', metadata,
            Column('wave_scale_answer_id', Integer, ForeignKey('tblTntCLMWaveScaleAnswer.id', ondelete="CASCADE"), primary_key=True),
            Column('locale_revision_id', Integer, ForeignKey('tblTntCLMWaveLocaleRevision.id'), primary_key=True),

            Column('label', UnicodeText, nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntCLMWaveAttributeLocale = IQTable('tblTntCLMWaveAttributeLocale', metadata,
            Column('wave_attribute_id', Integer, ForeignKey('tblTntCLMWaveAttribute.id', ondelete="CASCADE"), primary_key=True),
            Column('locale_revision_id', Integer, ForeignKey('tblTntCLMWaveLocaleRevision.id'), primary_key=True),

            Column('priority_trigger_at', SmallInteger, nullable=True),
            Column('label', UnicodeText, nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntCLMWaveAnswer = IQTable('tblTntCLMWaveAnswer', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('wave_attribute_id', Integer, ForeignKey('tblTntCLMWaveAttribute.id', ondelete="CASCADE"), nullable=False),
            Column('answer_id', Integer, ForeignKey('tblTntCLMAttributeAnswer.id', ondelete="RESTRICT"), nullable=False),

            Column('position', Integer, nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tblTntCLMWaveAnswerLocale = IQTable('tblTntCLMWaveAnswerLocale', metadata,
            Column('wave_answer_id', Integer, ForeignKey('tblTntCLMWaveAnswer.id', ondelete="CASCADE"), primary_key=True),
            Column('locale_revision_id', Integer, ForeignKey('tblTntCLMWaveLocaleRevision.id'), primary_key=True),

            Column('label', UnicodeText, nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntCLMWaveContactList = IQTable('tblTntCLMWaveContactList', metadata,
            Column('wave_id', Integer, ForeignKey('tblTntCLMWave.id'), primary_key=True),
            Column('contact_list_id', Integer, ForeignKey('tblContactList.id'), primary_key=True),
            Column('is_processed', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )
        tblTntCLMWaveMailing = IQTable('tblTntCLMWaveMailing', metadata,
            Column('wave_id', Integer, ForeignKey('tblTntCLMWave.id'), primary_key=True),
            Column('mailing_id', Integer, ForeignKey('tblMailing.id', ondelete='CASCADE'), primary_key=True)
        )

        tables['tblTntCLMAccount'] = tblTntCLMAccount
        tables['tblTntCLMAccountLocale'] = tblTntCLMAccountLocale
        tables['tblTntCLMCustomer'] = tblTntCLMCustomer
        tables['tblTntCLMContact'] = tblTntCLMContact
        tables['tblTntCLMCompany'] = tblTntCLMCompany
        tables['tblTntCLMService'] = tblTntCLMService
        tables['tblTntCLMLocale'] = tblTntCLMLocale
        tables['tblTntCLMWaveMailLocale'] = tblTntCLMWaveMailLocale
        tables['tblTntCLMWaveMail'] = tblTntCLMWaveMail
        tables['tblTntCLMWaveMainLabel'] = tblTntCLMWaveMainLabel
        tables['tblTntCLMWaveType'] = tblTntCLMWaveType
        tables['tblTntCLMWaveTypeGroup'] = tblTntCLMWaveTypeGroup
        tables['tblTntCLMScaleAnswer'] = tblTntCLMScaleAnswer
        tables['tblTntCLMScaleAnswerGroup'] = tblTntCLMScaleAnswerGroup
        tables['tblTntCLMScaleAnswerLocale'] = tblTntCLMScaleAnswerLocale
        tables['tblTntCLMAttribute'] = tblTntCLMAttribute
        tables['tblTntCLMAttributeGroup'] = tblTntCLMAttributeGroup
        tables['tblTntCLMAttributeLocale'] = tblTntCLMAttributeLocale
        tables['tblTntCLMAttributeAnswer'] = tblTntCLMAttributeAnswer
        tables['tblTntCLMAttributeAnswerLocale'] = tblTntCLMAttributeAnswerLocale
        tables['tblTntCLMWave'] = tblTntCLMWave
        tables['tblTntCLMWaveGroup'] = tblTntCLMWaveGroup
        tables['tblTntCLMWaveLocale'] = tblTntCLMWaveLocale
        tables['tblTntCLMWaveLocaleRevision'] = tblTntCLMWaveLocaleRevision
        tables['tblTntCLMWaveAttribute'] = tblTntCLMWaveAttribute
        tables['tblTntCLMWaveAttributeCategory'] = tblTntCLMWaveAttributeCategory
        tables['tblTntCLMWaveAttributeCategoryLocale'] = tblTntCLMWaveAttributeCategoryLocale
        tables['tblTntCLMWaveScaleAnswer'] = tblTntCLMWaveScaleAnswer
        tables['tblTntCLMWaveScaleAnswerLocale'] = tblTntCLMWaveScaleAnswerLocale
        tables['tblTntCLMWaveAttributeLocale'] = tblTntCLMWaveAttributeLocale
        tables['tblTntCLMWaveAnswer'] = tblTntCLMWaveAnswer
        tables['tblTntCLMWaveAnswerLocale'] = tblTntCLMWaveAnswerLocale
        tables['tblTntCLMWaveContactList'] = tblTntCLMWaveContactList
        tables['tblTntCLMWaveMailing'] = tblTntCLMWaveMailing


        __tables_defined = True

def do_mappings():
    """
    This function is only called by the django middleware
    when it is needed
    """
    global __mapping_defined
    if not __mapping_defined:
        from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
        from iqportal.iqube.core.hooks import MdxMemberMapper
        from basesoft.web.contrib.i18n import models as i18n_models
        from basesoft.web.contrib.mailing import models as mailing_models
        from iqportal.tnt.clm import models as clm_models
        from iqportal.tnt.core import models as core_models
        from basesoft.web.contrib.contact import models as contact_models
        from iqportal.tnt.response import models as response_models
        from basesoft.web.contrib.notification import models as notification_models
        from iqportal.tnt.core.sa_datamapper import get_tables as core_tables

        mapper(clm_models.TntCLMAccount, tables['tblTntCLMAccount'],
            properties = {
                'account': relation(account_models.Account),
                'geolevels': relation(core_models.TntGeolevel, secondary=core_tables()['tblTntGeolevelAccount'], order_by=core_models.TntGeolevel.type_id, passive_deletes=True),
                'locales': relation(clm_models.TntCLMLocale, secondary=tables['tblTntCLMAccountLocale'], passive_deletes=True),
            },
            polymorphic_identity='TntCLMAccount',
            inherits=account_models.Account
        )
        mapper(clm_models.TntCLMAccountLocale, tables['tblTntCLMAccountLocale'],
            properties = {}
        )

        mapper(clm_models.TntCLMCustomer, tables['tblTntCLMCustomer'],
            properties = {
                'country': relation(i18n_models.Country)
            }
        )

        mapper(clm_models.TntCLMContact, tables['tblTntCLMContact'],
            properties = {
                'contact': relation(contact_models.Contact)
            },
            polymorphic_identity='TntCLMContact',
            inherits=contact_models.Contact
        )

        mapper(clm_models.TntCLMCompany, tables['tblTntCLMCompany'],
            properties = {
                'company': relation(contact_models.Company),
                'locale': relation(clm_models.TntCLMLocale),
                'depot': relation(core_models.TntDepot),
                'customer': relation(clm_models.TntCLMCustomer)
            },
            polymorphic_identity='TntCLMCompany',
            inherits=contact_models.Company
        )

        mapper(clm_models.TntCLMNotification,
            polymorphic_identity='TntCLMNotification',
            inherits=notification_models.Notification
        )

        mapper(clm_models.TntCLMService, tables['tblTntCLMService'],
            extension = MdxMemberMapper(),
            properties = {}
        )

        mapper(clm_models.TntCLMLocale, tables['tblTntCLMLocale'],
            properties = {
                'locale': relation(i18n_models.Locale),
                'country': relation(i18n_models.Country),
                'service': relation(clm_models.TntCLMService, backref="locales"),
            }
        )

        mapper(clm_models.TntCLMWaveMailLocale, tables['tblTntCLMWaveMailLocale'], properties = {
           'mail': relation(mailing_models.Mail),
           'locale': relation(clm_models.TntCLMLocale)
        })

        mapper(clm_models.TntCLMWaveMail, tables['tblTntCLMWaveMail'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'mail': relation(mailing_models.Mail),
                'locales': relation(clm_models.TntCLMWaveMailLocale, order_by=tables['tblTntCLMWaveMailLocale'].c.locale_id, passive_deletes=True),
                'fallback_locale': relation(clm_models.TntCLMLocale),
            },
            polymorphic_identity='TntCLMWaveMail',
            inherits=mailing_models.Mail
        )

        mapper(clm_models.TntCLMWaveMainLabel, tables['tblTntCLMWaveMainLabel'])

        mapper(clm_models.TntCLMWaveType, tables['tblTntCLMWaveType'],
            properties = {
                'groups': relation(clm_models.TntCLMWaveTypeGroup, backref='type', cascade="all", passive_deletes=True),
            }
        )

        mapper(clm_models.TntCLMWaveTypeGroup, tables['tblTntCLMWaveTypeGroup'],
            properties = {
                'group': relation(license_models.Group)
            }
        )

        mapper(clm_models.TntCLMScaleAnswer, tables['tblTntCLMScaleAnswer'],
            properties = {
                'groups': relation(clm_models.TntCLMScaleAnswerGroup, backref='scale_answer', cascade="all", passive_deletes=True),
                'locales': relation(clm_models.TntCLMScaleAnswerLocale, passive_deletes=True),
                'type': relation(clm_models.TntCLMWaveType)
            }
        )

        mapper(clm_models.TntCLMScaleAnswerGroup, tables['tblTntCLMScaleAnswerGroup'],
            properties = {
                'group': relation(license_models.Group)
            }
        )

        mapper(clm_models.TntCLMScaleAnswerLocale, tables['tblTntCLMScaleAnswerLocale'],
            properties = {
                'scale_answer': relation(clm_models.TntCLMScaleAnswer),
                'locale': relation(clm_models.TntCLMLocale)
            }
        )

        mapper(clm_models.TntCLMAttribute, tables['tblTntCLMAttribute'],
            extension = [PrivilegeObjectMapper(), MdxMemberMapper()],
            properties = {
                'groups': relation(clm_models.TntCLMAttributeGroup, backref='attribute', cascade="all", passive_deletes=True),
                'locales': relation(clm_models.TntCLMAttributeLocale, passive_deletes=True),
                'answers': relation(clm_models.TntCLMAttributeAnswer)
            }
        )

        mapper(clm_models.TntCLMAttributeGroup, tables['tblTntCLMAttributeGroup'],
            properties = {
                'group': relation(license_models.Group)
            }
        )

        mapper(clm_models.TntCLMAttributeLocale, tables['tblTntCLMAttributeLocale'],
            properties = {
                'attribute': relation(clm_models.TntCLMAttribute),
                'locale': relation(clm_models.TntCLMLocale)
            }
        )

        mapper(clm_models.TntCLMAttributeAnswer, tables['tblTntCLMAttributeAnswer'],
            properties = {
                'attribute': relation(clm_models.TntCLMAttribute),
                'locales': relation(clm_models.TntCLMAttributeAnswerLocale, passive_deletes=True)
            }
        )

        mapper(clm_models.TntCLMAttributeAnswerLocale, tables['tblTntCLMAttributeAnswerLocale'],
            properties = {
                'answer': relation(clm_models.TntCLMAttributeAnswer),
                'locale': relation(clm_models.TntCLMLocale)
            }
        )

        mapper(clm_models.TntCLMWave, tables['tblTntCLMWave'],
            extension = [PrivilegeObjectMapper(), MdxMemberMapper()],
            properties = {
                'groups': relation(clm_models.TntCLMWaveGroup, backref='wave', cascade="all", passive_deletes=True),
                'locales': relation(clm_models.TntCLMWaveLocale, secondary=tables['tblTntCLMWaveLocaleRevision'],
                                    passive_deletes=True),
                'attributes': relation(clm_models.TntCLMWaveAttribute, order_by=tables['tblTntCLMWaveAttribute'].c.position),
                'categories': relation(clm_models.TntCLMWaveAttributeCategory),
                'type': relation(clm_models.TntCLMWaveType),
                'scale_answers': relation(clm_models.TntCLMWaveScaleAnswer, order_by=tables['tblTntCLMWaveScaleAnswer'].c.position),
                'contact_lists': relation(contact_models.ContactList, secondary=tables['tblTntCLMWaveContactList']),
                'mailings': relation(clm_models.TntCLMWaveMailing, passive_deletes=True),
                'invitees': relation(response_models.TntCLMWaveInvitee)
            }
        )

        mapper(clm_models.TntCLMWaveGroup, tables['tblTntCLMWaveGroup'],
            properties = {
                'group': relation(license_models.Group)
            }
        )

        mapper(clm_models.TntCLMWaveLocaleRevision, tables['tblTntCLMWaveLocaleRevision'],
            properties = {
                'wave': relation(clm_models.TntCLMWave),
                'locale': relation(clm_models.TntCLMLocale),
                'account': relation(account_models.Account),
            }
        )

        mapper(clm_models.TntCLMWaveLocale, tables['tblTntCLMWaveLocale'],
            properties = {
                'locale_revision': relation(clm_models.TntCLMWaveLocaleRevision)
            }
        )

        mapper(clm_models.TntCLMWaveAttribute, tables['tblTntCLMWaveAttribute'],
            properties = {
                'wave': relation(clm_models.TntCLMWave),
                'category': relation(clm_models.TntCLMWaveAttributeCategory),
                'attribute': relation(clm_models.TntCLMAttribute),
                'locales': relation(clm_models.TntCLMWaveAttributeLocale, passive_deletes=True),
                'answers': relation(clm_models.TntCLMWaveAnswer, cascade="all", passive_deletes=True,
                                    order_by=tables['tblTntCLMWaveAnswer'].c.position)
            }
        )

        mapper(clm_models.TntCLMWaveAttributeCategory, tables['tblTntCLMWaveAttributeCategory'],
            properties = {
                'wave': relation(clm_models.TntCLMWave),
                'locales': relation(clm_models.TntCLMWaveAttributeCategoryLocale, passive_deletes=True),
            }
        )

        mapper(clm_models.TntCLMWaveAttributeCategoryLocale, tables['tblTntCLMWaveAttributeCategoryLocale'],
            properties = {
                'category': relation(clm_models.TntCLMWaveAttributeCategory),
                'locale_revision': relation(clm_models.TntCLMWaveLocaleRevision)
            }
        )

        mapper(clm_models.TntCLMWaveScaleAnswer, tables['tblTntCLMWaveScaleAnswer'],
            properties = {
                'wave': relation(clm_models.TntCLMWave),
                'scale_answer': relation(clm_models.TntCLMScaleAnswer),
                'locales': relation(clm_models.TntCLMWaveScaleAnswerLocale, passive_deletes=True)
            }
        )

        mapper(clm_models.TntCLMWaveScaleAnswerLocale, tables['tblTntCLMWaveScaleAnswerLocale'],
            properties = {
                'wave_scale_answer': relation(clm_models.TntCLMWaveScaleAnswer),
                'locale_revision': relation(clm_models.TntCLMWaveLocaleRevision)
            }
        )

        mapper(clm_models.TntCLMWaveAttributeLocale, tables['tblTntCLMWaveAttributeLocale'],
            properties = {
                'wave_attribute': relation(clm_models.TntCLMWaveAttribute),
                'locale_revision': relation(clm_models.TntCLMWaveLocaleRevision)
            }
        )

        mapper(clm_models.TntCLMWaveAnswer, tables['tblTntCLMWaveAnswer'],
            properties = {
                'wave_attribute': relation(clm_models.TntCLMWaveAttribute),
                'answer': relation(clm_models.TntCLMAttributeAnswer),
                'locales': relation(clm_models.TntCLMWaveAnswerLocale, passive_deletes=True)
            }
        )

        mapper(clm_models.TntCLMWaveAnswerLocale, tables['tblTntCLMWaveAnswerLocale'],
            properties = {
                'wave_answer': relation(clm_models.TntCLMWaveAnswer),
                'locale_revision': relation(clm_models.TntCLMWaveLocaleRevision)
            }
        )

        mapper(clm_models.TntCLMWaveContactList, tables['tblTntCLMWaveContactList'],
            properties = {
                'wave': relation(clm_models.TntCLMWave),
                'contact_list': relation(contact_models.ContactList),
            }
        )

        mapper(clm_models.TntCLMWaveMailing, tables['tblTntCLMWaveMailing'],
            properties = {
                'wave': relation(clm_models.TntCLMWave),
            },
            polymorphic_identity='TntCLMWaveMailing',
            inherits=mailing_models.Mailing
        )

        __mapping_defined = True

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'TNT-CLM')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the `iqsyncpermissions` command.

    Titles are with gettext calls, but in the database they appear in English. We need it like this so we get the
    titles in the language files. Later, in the js interfaces we show them translated.
    """
    return [

        # Wave
        license_models.Permission(u'tnt_clm_wave_manager', u"""
            This gives the right to access the Wave Manager.
            """, _("Wave manager")),

        # Attribute
        license_models.Permission(u'tnt_clm_attribute_manager', u"""
            This gives the right to manage the Tnt CLM Attributes.
            """, _("Tnt CLM Attributes Manager")),
        license_models.Permission(u'create_tnt_clm_attribute', u"""
            This gives the right to create a a Tnt CLM Attribute.
            """, _("Create Tnt CLM Attributes")),
        license_models.Permission(u'view_tnt_clm_attribute', u"""
            This gives the right to view a Tnt CLM Attribute.
            """, _("View Tnt CLM Attributes")),
        license_models.Permission(u'update_tnt_clm_attribute', u"""
            This gives the right to update a Tnt CLM Attribute.
            """, _("Update Tnt CLM Attributes")),
        license_models.Permission(u'delete_tnt_clm_attribute', u"""
            This gives the right to delete a Tnt CLM Attribute.
            """, _("Delete Tnt CLM Attributes")),
        license_models.Permission(u'merge_tnt_clm_attribute', u"""
            This gives the right to merge Tnt CLM Attributes.
            """, _("Merge Tnt CLM Attributes")),
        license_models.Permission(u'translate_tnt_clm_attribute', u"""
            This gives the right to translate Tnt CLM Attributes.
            """, _("Translate attributes")),

        # Scale answer
        license_models.Permission(u'create_tnt_clm_scale_answer', u"""
            This gives the right to create a Tnt CLM Scale Answer.
            """, _("Create scale answer")),
        license_models.Permission(u'update_tnt_clm_scale_answer', u"""
            This gives the right to update a Tnt CLM Scale Answer.
            """, _("Update scale answer")),
        license_models.Permission(u'delete_tnt_clm_scale_answer', u"""
            This gives the right to delete a Tnt CLM Scale Answer.
            """, _("Delete scale answer")),
        license_models.Permission(u'view_tnt_clm_scale_answer', u"""
            This gives the right to view a Tnt CLM Scale Answer.
            """, _("View scale answer")),

        # wave
        license_models.Permission(u'create_tnt_clm_wave', u"""
            This gives the right to create a Tnt CLM Wave.
            """, _("Create wave")),
        license_models.Permission(u'update_tnt_clm_wave', u"""
            This gives the right to update a Tnt CLM Wave.
            """, _("Update wave")),
        license_models.Permission(u'delete_tnt_clm_wave', u"""
            This gives the right to delete a Tnt CLM Wave.
            """, _("Delete wave")),
        license_models.Permission(u'view_tnt_clm_wave', u"""
            This gives the right to view a Tnt CLM Wave.
            """, _("View wave")),
        license_models.Permission(u'translate_tnt_clm_wave', u"""
            This gives the right to translate a Tnt CLM Wave.
            """, _("Translate wave")),
        license_models.Permission(u'schedule_tnt_clm_wave', u"""
            This gives the right to schedule a Tnt CLM Wave.
            """, _("Schedule wave")),
        license_models.Permission(u'view_wave_responses', u"""
            This gives the right to view a wave responses.
            """, _("View wave responses")),
        license_models.Permission(u'view_tnt_clm_wave_translation_history', u"""
            This gives the right to view the translation history for Tnt CLM Waves.
            """, _("View wave translations history")),
        license_models.Permission(u'approve_tnt_clm_translations', u"""
            This gives the right to approve/reject translations for Tnt CLM Waves.
            """, _("Approve/reject wave translations")),
        license_models.Permission(u'set_wave_global_attribute_priority', u"""
            This gives the right to set a global value for wave attribute priority.
            """, _("Set wave global attribute priority")),


        # wave type
        license_models.Permission(u'create_tnt_clm_wave_type', u"""
            This gives the right to create a Tnt CLM Wave Type.
            """, _("Create wave type")),
        license_models.Permission(u'update_tnt_clm_wave_type', u"""
            This gives the right to update a Tnt CLM Wave Type.
            """, _("Update wave type")),
        license_models.Permission(u'delete_tnt_clm_wave_type', u"""
            This gives the right to delete a Tnt CLM Wave Type.
            """, _("Delete wave type")),
        license_models.Permission(u'view_tnt_clm_wave_type', u"""
            This gives the right to view a Tnt CLM Wave Type.
            """, _("View wave type")),

        # locale
        license_models.Permission(u'tnt_clm_locale_manager', u"""
            This gives the right to access the CLM Locale Manager.
            """, _("Locale tnt manager")),
        license_models.Permission(u'create_tnt_clm_locale', u"""
            This gives the right to create CLM Locales.
            """, _("Create tnt locale")),
        license_models.Permission(u'update_tnt_clm_locale', u"""
            This gives the right to update CLM Locales.
            """, _("Update tnt locale")),

        # contact
        license_models.Permission(u'tnt_clm_contact_access', u"""
            This gives the right to tnt clm specific contact manager.
            """, _("Tnt CLM Contact manager")),

        # tnt wave mail
        license_models.Permission(u"create_tnt_wave_mail", u"""
            This gives the right to create new tnt wave mails.
            """, _("Create tnt wave mails")),
        license_models.Permission(u"view_tnt_wave_mail", u"""
            This gives the right to view tnt wave mails.
            """, _("View tnt wave mails")),
        license_models.Permission(u"update_tnt_wave_mail", u"""
            This gives the right to update existing tnt wave mails.
            """, _("Update tnt wave mails")),
        license_models.Permission(u"delete_tnt_wave_mail", u"""
            This gives the right to delete tnt wave mails.
            """, _("Delete tnt wave mails")),
        license_models.Permission(u'approve_tnt_clm_wave_mail_translations', u"""
            This gives the right to approve/reject translations for Tnt CLM Wave mails.
            """, _("Approve/reject wave mail translations")),

        license_models.Permission(u'manage_tnt_clm_accounts', u"""
            This gives the rights to manage TNT CLM user accounts.
        """, _("Manage TNT CLM accounts.")),

    ]

def app_configs():
    """
    The account configs available for this application. This can be sync'd with the
    `iqsyncpermissions` command.
    """
    configs = []

    config = account_models.Config()
    config.name = 'tnt.clm.translate.locale.right'
    config.type = 'int'
    config.default = None
    config.description = u'TNT locale id for selected primary locale in translate interface.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'tnt.clm.translate.locale.left'
    config.type = 'int'
    config.default = None
    config.description = u'TNT locale id for selected secondary locale in translate interface.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'tnt.clm.mail.translate.locale.right'
    config.type = 'int'
    config.default = None
    config.description = u'TNT locale id for selected primary locale in mail translate interface.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'tnt.clm.mail.translate.locale.left'
    config.type = 'int'
    config.default = None
    config.description = u'TNT locale id for selected secondary locale in mail translate interface.'
    configs.append(config)

    return configs
