# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy.orm import join
from sqlalchemy import and_
from sqlalchemy.orm.exc import NoResultFound

from basesoft.web.contrib.license.models import Group, License, Domain

from basesoft.web.contrib.graph.db_sync import sync_color_sets
from basesoft.web.contrib.graph.models import ColorSet, GraphTheme
from django.utils import simplejson
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcJSONEncoder

from iqportal.iqreport.core.models import ReportTheme
from basesoft.web.contrib.graph.models import GraphTheme
from iqportal.iqreport.core.db_sync import sync_report_themes
from basesoft.web.contrib.branding.db_sync import _sync_themes

synced_apps = [] # global used to see what apps have already been synced until now

def sync(session):

    tnt_colors = ['#fe6403', '#4684EE', '#DC3912', '#FF9900', '#008000', '#666666',
                  '#4942CC', '#CB4AC5', '#d6ae00', '#336699', '#dd4477', '#aaaa11',
                  '#66aa00', '#888888', '#994499', '#dd5511', '#22aa99', '#999999',
                  '#705770', '#109618', '#a32929']

    # nothing to sync if no TNT domain
    tnt_domain = None
    try:
        tnt_domain = session.query(Domain).filter(Domain.name=='TNT').one()
        # create the color sets that are used for reportlab charts for tnt
        group = tnt_domain.default_group
        domain_privs = ColorSet.PRIV_VIEW
    except NoResultFound:
        print 'not syncing the tnt color theme: tnt license not available '
        return

    color_sets = [
        {
            'name': u'TNT',
            'color_config': simplejson.dumps(tnt_colors, ensure_ascii=False, cls=JsonRpcJSONEncoder),
            'group': group,
            'group_privileges': ColorSet.all_privileges,
            'license_privileges': ColorSet.no_privileges,
            'domain_privileges': domain_privs,
            'world_privileges': ColorSet.no_privileges
        }
    ]
    sync_color_sets(session, color_sets)

    # create the graph themes for TNT
    tnt_graphs = {
        'chart_background_top_color': '#fef5ea',
        'overlap_legend': False,
        'legend_width': 100,
        'legend_render': 'columns',
        'legend_colsize': 1,
        'legend_border_color': '#F97000',
        'show_chart_border': True,
        'chart_border_color': '#F97000',
    }

    default_graph_themes = [{
        'name': 'TNT',
        'module': 'graph_theme_tnt',
        'css_class': 'graph-tnt',
        'graph_config': simplejson.dumps(tnt_graphs, ensure_ascii=False, cls=JsonRpcJSONEncoder),
        'default_color_set': session.query(ColorSet).filter(ColorSet.name=='TNT').one(),
        'group': group,
        'group_privileges': GraphTheme.all_privileges,
        'license_privileges': GraphTheme.no_privileges,
        'domain_privileges': GraphTheme.no_privileges,
        'world_privileges': GraphTheme.PRIV_VIEW
    }]

    current_themes = [t.name for t in session.query(GraphTheme).all()]
    for theme in default_graph_themes:

        # if we already have a theme with such a name, skip it for now.
        if theme['name'] in current_themes:
            continue

        t = GraphTheme()
        for attr in theme.keys():
            setattr(t, attr, theme[attr])
        session.add(t)
    session.flush()
    # end of create graph themes

def post_sync(session, app_name):
    """
    Sync objects only after app_name has ran it's sync()
    @param session: the sqlalchemy session
    @param app_name: the name of the app that has been sync-ed
    """
    
    try:
        session.query(Domain).filter(Domain.name=='TNT').one()
    except NoResultFound:
        print 'post_sync: tnt license not available '
        return    
    
    global synced_apps
    synced_apps.append(app_name)

    if ('basesoft.web.contrib.graph' in synced_apps) and ('iqportal.tnt.core' in synced_apps):
        # sync themes only after color_sets have been created (also the ones from this app's sync())
        themes = [{
            'name': u'TNT',
            'title': u'TNT - IQ OES',
            'module_name': u'tnt_theme_default',
            'css_class': u'tnt',
            'licenses': ['TNT CLM'],
            'default': 'TNT CLM',
            'default_graph_theme': u'Default',
        }]
        _sync_themes(session, themes) # create the TNT theme even if there's no TNT domain (can be used for demo purposes)

        sync_report_themes(session, [{
            'name': 'TNT',
            'module': None,
            'css_class': None,
            'graph_theme': session.query(GraphTheme).filter(GraphTheme.name=='TNT').one(),
            'group_privileges': ReportTheme.all_privileges,
            'license_privileges': ReportTheme.no_privileges,
            'domain_privileges': ReportTheme.no_privileges,
            'world_privileges': 0
        }])
