# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon

class TntGeolevelManager(IconBarIcon):
    """
    Opens the interface for geolevel overview manager
    """

    id = 'icon-tnt-geolevel-manager'
    title = _('TNT Geolevels')
    icon_index = 83
    component = 'tnt_geolevel_manager'
    group = 'internationalization'
    colorname = 'iconbar-rust'

    char_main = u'\uE800' # World icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('geolevel_manager')

class TntDepotManager(IconBarIcon):
    """
    Opens the interface for depot overview manager
    """

    id = 'icon-tnt-depot-manager'
    title = _('TNT Depots')
    icon_index = 84
    component = 'tnt_depot_manager'
    group = 'internationalization'
    colorname = 'iconbar-rust'

    char_main = u'\uE800' # World icon
    char_sub = u'\uE839' # Arrow icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('depot_manager')

def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """

    return [TntGeolevelManager(), TntDepotManager()]
