# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _
from basesoft.web.core.models import PrivilegeObject

class TntGeolevelType(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<TntGeolevelType id="%s">' % self.id

class TntGeolevel(PrivilegeObject):
    """
    @DynamicAttrs
    """

    mark_as_deleted_permission = 'delete_geolevel'
    mark_as_archived_permission = 'update_geolevel'

    _name_property = 'name'

    def get_mdx_member_config(self):
        rv = [{
            'datasource': 'TNT-CLM',
            'cube': 'TntCLMResponse',
            'prefix': '[Country]',
            'field': 'name',
        }]
        return rv

    def get_group_object(self):
        return (TntGeolevelGroup, 'geolevel')

    def get_countries(self):
        countries = self.countries
        for child in self.child_geolevels:
            [countries.append(c) for c in child.get_countries() if c not in countries]
        return countries

    def __repr__(self):
        return '<TntGeolevel id="%s">' % self.id

class TntGeolevelGeolevel(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<TntGeolevelGeolevel>'

class TntGeolevelGroup(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<TntGeolevelGroup geolevel_id=%s group_id=%s>' % (self.geolevel_id, self.group_id)

class TntGeolevelCountry(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<TntGeolevelCountry>'


class TntGeolevelAccount(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<TntGeolevelAccount>'

class TntDepot(PrivilegeObject):
    """
    @DynamicAttrs
    """
    cols_i18n = {
        "default_code": _("Depot code")
    }

    mark_as_deleted_permission = 'delete_depot'
    mark_as_archived_permission = 'update_depot'

    _name_property = 'default_code'

    def get_mdx_member_config(self):
        rv = [{
            'datasource': 'TNT-CLM',
            'cube': 'TntCLMResponse',
            'prefix': '[Depot]',
            'field': 'default_code',
        }]
        return rv

    def get_group_object(self):
        return (TntDepotGroup, 'depot')

    def __repr__(self):
        return '<TntDepot id="%s">' % self.id

class TntDepotAlias(object):
    """
    @DynamicAttrs
    """
    def get_mdx_member_config(self):
        rv = [{
            'datasource': 'TNT-CLM',
            'cube': 'TntCLMResponse',
            'prefix': '[Depot]',
            'field': 'code',
        }]
        return rv

    def __repr__(self):
        return '<TntDepotAlias>'

class TntDepotGroup(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<TntDepotGroup>'
