# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.
from iqportal.tnt.core.controllers import TntController
from basesoft.web.contrib.i18n.controllers import I18nController

class GeolevelRPCHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.controller = TntController(request.Session(), account_id=request.session['user_id'])

    def get_geolevels(self):
        rv = []
        for geo in self.controller.get_geolevels():
            rv.append({
                'value': geo.id,
                'label': u'[%s] %s' % (geo.type.name, geo.name)
            })
        return rv

    def get_geolevel_types(self):
        return [{'value': geo.id, 'label': geo.name} for geo in self.controller.get_geolevel_types_by()]

    def get_child_geolevels(self, geolevel_id=None):
        """
        Returns childs of the provided geolevel_id, or root geolevels when geolevel_id is None
        """
        i18n_controller = I18nController(self.controller.sa_session, account_id=self.controller.get_account().id)
        if geolevel_id is not None:
            g = self.controller.get_geolevels(geolevel_id)[0]
            geolevels = g.child_geolevels
            cids = [c.id for c in g.countries]
            countries = [country.name if domain_country is None else domain_country.name
                         for country, domain_country in i18n_controller.get_domain_countries(cids)]
        else:
            geolevels = [g for g in self.controller.get_geolevels() if len(g.parent_geolevels) == 0]
            countries = []
        return {'geolevels': [self.controller.load_geolevel_state(g) for g in geolevels],
                'countries': countries}

handlers = (GeolevelRPCHandler, )
