# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

import datetime

from sqlalchemy.orm import mapper, relation
from sqlalchemy import Column, Integer, ForeignKey, DateTime, UnicodeText, Unicode, SmallInteger
from django.utils.translation import ugettext as _

from basesoft.sqlalchemy.table import IQTable
from basesoft.web.contrib.license import models as license_models

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    """
    """
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        tblTntGeolevelType = IQTable('tblTntGeolevelType', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', Unicode(255), nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntGeolevel = IQTable('tblTntGeolevel', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', UnicodeText, nullable=False),
            Column('type_id', Integer, ForeignKey('tblTntGeolevelType.id'), nullable=False),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntGeolevelGeolevel = IQTable('tblTntGeolevelGeolevel', metadata,
            Column('parent_geolevel_id', Integer, ForeignKey('tblTntGeolevel.id', ondelete='CASCADE'), primary_key=True),
            Column('geolevel_id', Integer, ForeignKey('tblTntGeolevel.id', ondelete='CASCADE'), primary_key=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntGeolevelGroup = IQTable('tblTntGeolevelGroup', metadata,
            Column('geolevel_id', Integer, ForeignKey('tblTntGeolevel.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntGeolevelCountry = IQTable('tblTntGeolevelCountry', metadata,
            Column('geolevel_id', Integer, ForeignKey('tblTntGeolevel.id', ondelete='CASCADE'), primary_key=True),
            Column('country_id', Integer, ForeignKey('tblCountry.id'), primary_key=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntGeolevelAccount = IQTable('tblTntGeolevelAccount', metadata,
            Column('geolevel_id', Integer, ForeignKey('tblTntGeolevel.id', ondelete='CASCADE'), primary_key=True),
            Column('account_id', Integer, ForeignKey('tblAccount.id', ondelete='CASCADE'), primary_key=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntDepot = IQTable('tblTntDepot', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('country_id', Integer, ForeignKey('tblCountry.id'), nullable=False),
            Column('service_id', Integer, ForeignKey('tblTntCLMService.id'), nullable=True, default=None),

            Column('default_code', Unicode(255), nullable=False),
            Column('label', Unicode(255), nullable=True),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tblTntDepotAlias = IQTable('tblTntDepotAlias', metadata,
            Column('depot_id', Integer, ForeignKey('tblTntDepot.id', ondelete="CASCADE"), primary_key=True),
            Column('code', Unicode(255), nullable=False, primary_key=True),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTntDepotGroup = IQTable('tblTntDepotGroup', metadata,
            Column('depot_id', Integer, ForeignKey('tblTntDepot.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tables['tblTntGeolevelType'] = tblTntGeolevelType
        tables['tblTntGeolevel'] = tblTntGeolevel
        tables['tblTntGeolevelGeolevel'] = tblTntGeolevelGeolevel
        tables['tblTntGeolevelGroup'] = tblTntGeolevelGroup
        tables['tblTntGeolevelCountry'] = tblTntGeolevelCountry
        tables['tblTntGeolevelAccount'] = tblTntGeolevelAccount
        tables['tblTntDepot'] = tblTntDepot
        tables['tblTntDepotAlias'] = tblTntDepotAlias
        tables['tblTntDepotGroup'] = tblTntDepotGroup

        __tables_defined = True


def do_mappings():
    """
    This function is only called by the django middleware
    when it is needed
    """
    global __mapping_defined
    if not __mapping_defined:
        from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
        from iqportal.iqube.core.hooks import MdxMemberMapper
        from basesoft.web.contrib.i18n import models as i18n_models
        from basesoft.web.contrib.account import models as account_models
        from iqportal.tnt.core import models as core_models
        from iqportal.tnt.clm import models as clm_models # uff, move services some day?

        mapper(core_models.TntGeolevelType, tables['tblTntGeolevelType'])

        mapper(core_models.TntGeolevel, tables['tblTntGeolevel'],
            extension = [PrivilegeObjectMapper(), MdxMemberMapper()],
            properties = {
                'type': relation(core_models.TntGeolevelType),
                'groups': relation(core_models.TntGeolevelGroup, backref='geolevel', cascade="all", passive_deletes=True),
                'countries': relation(i18n_models.Country, secondary=tables['tblTntGeolevelCountry'], backref="geolevels"),
                'accounts': relation(account_models.Account, secondary=tables['tblTntGeolevelAccount'], passive_deletes=True),
                'child_geolevels': relation(core_models.TntGeolevel, secondary=tables['tblTntGeolevelGeolevel'],
                    primaryjoin=tables['tblTntGeolevel'].c.id==tables['tblTntGeolevelGeolevel'].c.parent_geolevel_id,
                    secondaryjoin=tables['tblTntGeolevelGeolevel'].c.geolevel_id==tables['tblTntGeolevel'].c.id,
                    backref="parent_geolevels")
            }
        )

        mapper(core_models.TntGeolevelGeolevel, tables['tblTntGeolevelGeolevel'])

        mapper(core_models.TntGeolevelGroup, tables['tblTntGeolevelGroup'],
            properties = {
                'group': relation(license_models.Group)
            }
        )

        mapper(core_models.TntGeolevelCountry, tables['tblTntGeolevelCountry'],
            properties = {
                'geolevel': relation(core_models.TntGeolevel),
                'country': relation(i18n_models.Country)
            }
        )

        mapper(core_models.TntGeolevelAccount, tables['tblTntGeolevelAccount'],
            properties = {
                'geolevel': relation(core_models.TntGeolevel),
                'account': relation(account_models.Account)
            }
        )

        mapper(core_models.TntDepot, tables['tblTntDepot'],
            extension = [PrivilegeObjectMapper(), MdxMemberMapper()],
            properties = {
                'groups': relation(core_models.TntDepotGroup, backref='depot', cascade="all", passive_deletes=True),
                'aliases': relation(core_models.TntDepotAlias, backref='alias', cascade="all", passive_deletes=True),
                'country': relation(i18n_models.Country),
                'service': relation(clm_models.TntCLMService),
            }
        )

        mapper(core_models.TntDepotAlias, tables['tblTntDepotAlias'],
            extension = MdxMemberMapper(),
            properties = {
                'depot': relation(core_models.TntDepot)
            }
        )

        mapper(core_models.TntDepotGroup, tables['tblTntDepotGroup'],
            properties = {
                'group': relation(license_models.Group)
            }
        )

        __mapping_defined = True

def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'TNT')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.

    Titles are with gettext calls, but in the database they appear in English. We need it like this so we get the
    titles in the language files. Later, in the js interfaces we show them translated.
    """
    return [
        license_models.Permission(u"create_geolevel", u"""
            Allow the user to create a new geolevel. This is only a global permission and is not
            related to the different types of geolevels like TntRegion, TntBusinessUnit, etc.
        """, _("Create geolevels")),
        license_models.Permission(u"view_geolevel", u"""
            Allow the user to view an existing geolevel. This is only a global permission and is not
            related to the different types of geolevels like TntRegion, TntBusinessUnit, etc.
        """, _("View geolevels")),
        license_models.Permission(u"update_geolevel", u"""
            This gives the rights to update an existing geolevel. This is only a global permission
            and is not related to the different types of geolevels like TntRegion, TntBusinessUnit, etc.
        """, _("Update geolevels")),
        license_models.Permission(u"delete_geolevel", u"""
            This gives the rights to delete a geolevel. This is only a global permission and is not
            related to the different types of geolevels like TntRegion, TntBusinessUnit, etc.
        """, _("Delete geolevels")),
        license_models.Permission(u"geolevel_manager", u"""
            This gives the rights to the geolevel manager interface. From there, the user can manage the
            geolevels he has access to.
        """, _("Geolevel manager")),

        license_models.Permission(u"view_depot", u"""
            This gives the rights to view existing depots.
        """, _('View depots')),
        license_models.Permission(u"create_depot", u"""
            This gives the rights to create new depots.
        """, _('Create depots')),
        license_models.Permission(u"update_depot", u"""
            This gives the rights to update existing depots.
        """, _('Update depots')),
        license_models.Permission(u"delete_depot", u"""
            This gives the rights to delete existing depots.
        """, _('Delete depots')),
        license_models.Permission(u"depot_manager", u"""
            This gives the rights to the depot manager interface.
        """, _('Depots manager')),

        license_models.Permission(u"depot_alias_manager", u"""
            This gives the rights to manage depots aliases.
        """, _('Manage depot aliases')),
    ]
