# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher

from iqportal.tnt.core.controllers import TntController
from basesoft.web.contrib.i18n.controllers import I18nController

def rpc(request):
    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(Handler(request.Session(), request.session['user_id']))
    return dispatcher.handle_request(request)

class Handler:
    """
    Handles rpc calls for the geolevels.
    """

    def __init__(self, sa_session, account_id):
        self.sa_session = sa_session
        self.account_id = account_id
        self.controller = TntController(sa_session, None, account_id)
        self.i18n_controller = I18nController(sa_session, None, account_id)

    def save_geolevel(self, state):
        """
        Save the geolevel and return the id of the object

        State example:
            {
                dbID: None,
                name: u'Benelux',
                typeName: u'TntBusinessUnit',
                countries: [22, 155, 128], # ids of the countries: Belgium, Netherlands, Luxembourg
                accounts: []
            }
        """
        g = self.controller.save_geolevel_state(state)
        return g.id # we want only the id of the first element in the list (we only saved one product)

    def load_geolevel(self, geolevel_id):
        """
        Load the geolevel state and send it to the client
        """
        if geolevel_id is None:
            return {}

        return self.controller.load_geolevel_state(geolevel_id)

    def load_geolevel_data(self, geolevel_id):
        """
        Load the geolevel data and send it to the client
        """
        if geolevel_id is None:
            state = {}
        else:
            state = self.controller.load_geolevel_state(geolevel_id)

        allg = self.controller.get_geolevels()
        if geolevel_id is not None:
            gl = [g for g in allg if g.id==geolevel_id][0]
            parentg = []
            for g in allg:
                if g.id == geolevel_id:
                    continue
                if not self.controller.isChildOf(g.id, gl):
                    parentg.append(self.controller.load_geolevel_state(g))
        else:
            parentg = [self.controller.load_geolevel_state(g) for g in allg]


        return {
            'geolevel_state': state,
            'available_parent_geolevels': parentg
        }

    def get_geolevels(self, type=None):

        return [{'value': g.id, 'label': g.name} for g in self.controller.get_geolevels()]

    def save_depot(self, state):
        """
        Save the depot and return the id of the object

        State example:
            {
                dbID: None,
                default_name: u'Home Depot',
                country: 22# id of Belgium
            }
        """
        d = self.controller.save_depot_state(state)
        return d.id # we want only the id of the first element in the list (we only saved one depot)

    def delete_geolevels(self, id):
        """
        Delete the geolevels with the given id(s).
        @param id: int or list of ints
        """
        self.controller.delete_geolevels(id)

    def delete_depots(self, id):
        """
        Delete the depots with the given id(s).
        @param id: int or list of ints
        """
        self.controller.delete_depots(id)

    def load_depot(self, depot_id):
        """
        Load the depot state and send it to the client
        """
        if depot_id is None:
            return {}

        return self.controller.load_depot_state(depot_id)

    def get_depots_for_country(self, country_id=None):
        if country_id is None:
            return []

        return [{'label': d.default_code, 'value': d.id} for d in self.controller.get_depots(country_id=country_id)]

    def get_depots(self):
        return [{'label': d.default_code, 'value': d.id} for d in self.controller.get_depots()]
