# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from iqportal.iqube.xmla import client as iqube_client, mdx

class MdxBuilder(mdx.MdxBuilder):
    found_answer_dimension = None

    def _modify_hierarchies(self, hierarchies, union, order, axis_type):
        """
        This will modify hierarchies for CLM Specific data
        """

        if not self.is_graph:
            self.__process_total_answers(hierarchies, union, order, axis_type)

        if self.cube == 'TntCLMTrueResponseCube':
            self.__process_geolevel_children(hierarchies, union, order, axis_type)
        else:
            self.__process_pool_members(hierarchies, union, order, axis_type)

    def __process_geolevel_children(self, hierarchies, union, order, axis_type):
        """
        TODO: we filter in the reporter now, however that only works for widgets.
        """
        return

        from basesoft.web.core.exceptions import SystemException

        if self.account_id is None:
            raise SystemException('TNT-CLM::TntCLMTrueResponseCube should be initialized with an account_id')

        # no need to process if we dont use geolevels
        if not hierarchies.has_key('Geolevel'):
            return

        from basesoft.web.core.util import db
        from basesoft.web.contrib.account.models import Account

        # because the geolevel hierarchy is not a True hierarchy, members can be duplicates.
        sa_session = db.Session()
        account = sa_session.query(Account).filter(Account.id==self.account_id).one()

        # we select children based on unique id.


        return

    def __process_total_answers(self, hierarchies, union, order, axis_type):

        if axis_type == self.SLICER:
            return

        if self.found_answer_dimension is True:
            return

        if self.found_answer_dimension is None and not hierarchies.has_key('[KPI]'):
            self.found_answer_dimension = False
            return

        if not hierarchies.has_key('[KPI]'):
            hierarchies['[KPI]'] = {}
            hierarchies['[KPI]']['depth_0'] = ['[KPI].[No. of Answers]']
            self.found_answer_dimension = True
            return

        for depth in hierarchies['[KPI]'].keys():
            #if '[KPI].[No. of Answers]' in members:
            #    self.found_answer_dimension = True
            #    return
            hierarchies['[KPI]'][depth].append('[KPI].[No. of Answers]')
            self.found_answer_dimension = True
            return

    def __process_pool_members(self, hierarchies, union, order, axis_type):
        # Pool attributes are added as scale attributes. If we have no Pool attributes, skip this modification
        # We also do not change this if they are part of the slicer axis.
        if not hierarchies.has_key('[Pool]') or axis_type == self.SLICER:
            return

        is_union = False

        if hierarchies.has_key('[Attribute].[Scale]'):
            is_union = True
        else:
            hierarchies['[Attribute].[Scale]'] = {}

        for depth, members in hierarchies['[Pool]'].iteritems():
            if not hierarchies['[Attribute].[Scale]'].has_key(depth):
                hierarchies['[Attribute].[Scale]'][depth] = []
            elif is_union:
                new = ['{%s}' % ', '.join(hierarchies['[Attribute].[Scale]'][depth])]
                hierarchies['[Attribute].[Scale]'][depth] = new

            hierarchies['[Attribute].[Scale]'][depth].append('EXTRACT(FILTER(CROSSJOIN({%s}, {[Attribute].[Scale].Children}), NOT ISEMPTY([KPI].[No. of Answers])), [Attribute])' % ', '.join(members))
            if len(hierarchies['[Attribute].[Scale]'][depth])==1 or not is_union:
                continue

            hierarchies['[Attribute].[Scale]'][depth] = ['UNION(%s)' % ', '.join(hierarchies['[Attribute].[Scale]'][depth])]

        if '[Attribute].[Scale]' not in order:
            order[order.index('[Pool]')] = '[Attribute].[Scale]'

        if '[Pool]' in order:
            order[order.index('[Pool]'):order.index('[Pool]')+1] = []

class XMLAClient(iqube_client.XMLAClient):
    def set_mdx_builder(self):
        self._mdx_builder = MdxBuilder
