# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _
from sqlalchemy.orm.exc import NoResultFound
from iqportal.tnt.report.configurable_widgets.respondents_widget import RespondentsWidget
from iqportal.tnt.report.configurable_widgets.loyalty_league_table_widget import LoyaltyLeagueTableNewWidget
from iqportal.tnt.report.configurable_widgets.satisfaction_kpi_graph_widget import SatisfactionKPIgraphWidget
from iqportal.tnt.report.configurable_widgets.satisfaction_table_widget import NewSatisfactionTableWidget
from iqportal.tnt.report.configurable_widgets.diff_and_avg_widget import DiffandAvgWidget
from iqportal.tnt.report.configurable_widgets.nps_kpi_widget import NpsKpiWidget
from iqportal.tnt.report.configurable_widgets.general_attributes_widget import GeneralAttributeWidget

def sync(session):
    """
    Synchronizes the view for Tnt CLM respondents, adds custom datatablecube for TNT

    """
    from django.conf import settings
    from sqlalchemy.sql import text
    from sqlalchemy import create_engine
    from iqportal.iqreport.core.models import IQReportGraphType

    graph_types = [{
        'module': 'tnt_graph_datatablecube',
        'name': 'tnt_graph_datatablecube',
        'label': _('Tnt datatablecube'),
        'is_always_available': False
    }]
    current_types = [t.name for t in session.query(IQReportGraphType).all()]
    for gt in graph_types:
        if gt['name'] in current_types:
            continue

        type = IQReportGraphType()
        for attr, value in gt.iteritems():
            setattr(type, attr, value)
        session.add(type)

    session.flush()

    db_engine = create_engine(settings.DB_CONNSTR, echo=False)

    # TNT specific country view
    db_engine.execute(text("""
        DROP VIEW IF EXISTS viewTntCountry
    """))
    db_engine.execute(text("""
        CREATE VIEW viewTntCountry (country_id, name, code)
        AS SELECT c.id as country_id, IFNULL(dc.name, c.name) as name, c.iso_3166_a2 as code
        FROM tblCountry as c
        LEFT JOIN tblDomainCountry as dc
            ON c.id = dc.country_id
                AND dc.domain_id = (SELECT id FROM tblDomain WHERE NAME='TNT')
        WHERE (dc.active IS NULL OR dc.active = 1)
    """))

    db_engine.execute(text("""
        DROP VIEW IF EXISTS viewTntCLMRespondents
    """))

    db_engine.execute(text("""
        CREATE VIEW viewTntCLMRespondents (wave_id, locale_id, country_id, service_id, depot_id, participant_id, response_year, response_quarter, response_month, response_day, response_week, response_day_of_week, response_hour, response_minute)
        AS SELECT i.wave_id, comp.locale_id, l.country_id, l.service_id, comp.depot_id, p.id, year(max(ps.created)), quarter(max(ps.created)), monthname(max(ps.created)), day(max(ps.created)), week(max(ps.created)), dayname(max(ps.created)), hour(max(ps.created)), minute(max(ps.created))
        FROM tblTntCLMWaveResponse as r
        INNER JOIN tblTntCLMWaveParticipant as p
            ON r.participant_id = p.id
        INNER JOIN tblTntCLMWaveParticipantSession as ps
            ON p.id = ps.participant_id
        INNER JOIN tblTntCLMWaveInvitee as i
            ON p.invitee_id = i.id
        INNER JOIN tblContact as c
            ON i.contact_id = c.id
        INNER JOIN tblTntCLMCompany as comp
            ON c.company_id = comp.company_id
        INNER JOIN tblTntCLMLocale as l
            ON comp.locale_id = l.id
        INNER JOIN tblTntCLMWaveLocaleRevision as wlr
            ON i.wave_id = wlr.wave_id
        INNER JOIN tblTntCLMWaveLocale as wl
            ON wl.is_default = 1
                AND wlr.id = wl.locale_revision_id
        GROUP BY p.id
        ORDER BY wlr.id DESC
    """))

# fixme: enable: CrossReferenceWidget, GeneralAttributeGraphWidget

configurable_widgets = (LoyaltyLeagueTableNewWidget, SatisfactionKPIgraphWidget, NewSatisfactionTableWidget,
                        DiffandAvgWidget, RespondentsWidget, NpsKpiWidget, GeneralAttributeWidget)

