# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _

from basesoft.web.contrib.account.models import Account
from basesoft.util.ordered_dict import OrderedDict
from basesoft.web.contrib.i18n.controllers import I18nController
from basesoft.web.core.util.controllers import StateManager

from iqportal.tnt.report.controllers import TntBusinessFunctionController
from iqportal.tnt.clm.models import TntCLMAttribute
from iqportal.tnt.clm.controllers import TntCLMAttributeController, WaveController
from iqportal.tnt.core.controllers import TntController
from iqportal.iqreport.core.rpc import ResultHandler
from iqportal.tnt.report.reporters import TntCLMSatisfactionReporter, TntCLMSatisfactionGraphsReporter, TntCLMCellClickReporter, \
    TntCLMLoyaltyScoreReporter, TntCLMActionPlanningReport, TntCLMNCSLeagueTableReport, TntCLMLoyaltyLeagueTableReport, TntCLMGeneralQuestionsReporter

class BusinessFunctionHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self._controller = TntBusinessFunctionController(self.sa_session, account_id=request.session['user_id'])

    def save_or_update_business_function(self, id, cube, values):
        self._controller.save_or_update_business_function(id, cube, **values)
        return True

class ReportHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.account_id = request.session['user_id']

    def _get_business_functions(self, dimensions):
        bfc = TntBusinessFunctionController(self.sa_session, account_id=self.account_id)
        #return [{'value': "MEMBER %s.[%s] AS '%s'" % (bf.dimension, bf.name, bf.formula), 'label': bf.name} for bf in bfc.get_business_functions_by_dimension(dimensions)]
        return [{'value': bf.id, 'label': bf.label} for bf in bfc.get_business_functions_by_dimension(dimensions)]

    def get_attribute_options_bu(self, as_options=True):
        ac = TntCLMAttributeController(self.sa_session, StateManager(None, self.account_id, None))
        attributes = [a for a in ac.get_attributes_by_type(TntCLMAttribute.TYPE_SCALE)]

        # hackity smackity for General Question b
        [attributes.append(a) for a in ac.get_attributes(32)]

        if not as_options:
            return attributes

        return [{'value': a.id, 'label': (a.default_label not in [None, '']) and a.default_label or _('Country Question')}
                for a in attributes]

    def get_attribute_options(self):
        """
        Return a list with Business Functions and Scale Attributes to be selectable.
        """
        bf = self._get_business_functions('Position')
        bf.append({'value': '[Position].[Overall Satisfaction]', 'label': 'Overal Satisfaction'})

        attributes = [{'value': '[Pool].[%s]' % a.id,
                       'label': (a.default_label not in [None, '']) and a.default_label or _('Country Question')}
                      for a in self.get_attribute_options_bu(False)]

        return {
            _('Satisfaction/Cluster'): bf,
            _('Attribute'): attributes
        }

    def get_wave_options(self):

        rv = OrderedDict([
            ('All', [
                {'value': '[Wave].[All Waves]', 'label': 'All Waves'},
                {'value': '[Wave].[Code].Members', 'label': 'All Waves (Children)'},
            ]),
            (_('Main Waves'), [
                {'value': '[Wave].[Main Waves]', 'label': 'Main Waves'},
                {'value': '[Wave].[Main Waves].Children', 'label': 'Main Waves (Children)'},
            ]),
            (_('Other'), [
                {'value': '[Wave].[Other Waves]', 'label': 'Other Waves'},
                {'value': '[Wave].[Other Waves].Children', 'label': 'Other Waves (Children)'},
            ])
        ])

        wc = WaveController(self.sa_session, account_id=self.account_id)

        waves = wc.get_waves()
        waves.reverse()

        for w in waves:
            v = {'value': '[Wave].[Code].[%s]' % w.code, 'label': w.code}
            if w.is_main:
                rv[_('Main Waves')].append(v)
            else:
                rv[_('Other')].append(v)

        return rv

    def get_answer_options(self):

        answers = [
            'No. of Answers',
            'Dissatisfied',
            'Meeting Cust. Exp.',
            'Exceeding Cust. Exp.',
            'NCS Score',
            'Mean Score',
            'Much lower',
            'Lower',
            'As',
            'Higher',
            'Much higher',
            '% Much lower',
            '% Lower',
            '% As',
            '% Higher',
            '% Much higher',
        ]

        return {
        #    _('Business functions'): self._get_business_functions(['[Answers]']),
            _('KPI'): [{'value': '[KPI].[%s]' % a, 'label': a} for a in answers]
        }

    def get_loyalty_answer_options(self):
        return {
            _('KPI'): [{'value': '[KPI].[%s]' % a, 'label': a} for a in ['No. of Answers', 'Score', 'Score %']]
        }

    def get_geolevel_options(self):

        tnt_c = TntController(self.sa_session, account_id=self.account_id)
        i18n_c = I18nController(self.sa_session, account_id=self.account_id)

        rv = OrderedDict([
            ('All', [
                {'value': "[Country].[All Countries]", 'label': 'All Countries'},
                {'value': "[Country].[All Countries].Children", 'label': 'All Countries (Children)'},
            ]),

            # hack but they want this specific order:
            ('World', []),
            ('BusinessUnit', []),
            ('Region', []),
            ('Subregion', []),

            # othher geolevles will go here
            # note that 'Countries' are always last

        ])


        for geo_type in tnt_c.get_geolevel_types_by():

            rv[geo_type.name] = []
            for geol in tnt_c.get_geolevels(type_id=geo_type.id):

                #geo_cm = '+'.join(['[Country].[%s]' % country.iso_3166_a2 for country in geol.get_countries()])
                #value = "MEMBER [Country].[%s] AS '%s'" % (geol.name, geo_cm )

                value = "[Country].[%s]"  % geol.name

                rv[geo_type.name].append({
                    'value': value,
                    'label': geol.name
                })


        account = self.sa_session.query(Account).filter(Account.id==self.account_id).one()

        # union countries
        rv[_('Countries')] = [{
            'value': '[Country].[%s]' % country.iso_3166_a2,
            'label': country.name if domain_country is None else domain_country.name,
            'default': False} for country, domain_country in i18n_c.get_domain_countries()
                              if not hasattr(account, 'countries') or country in account.countries]


        return rv

    def get_depot_options(self):
        tnt_c = TntController(self.sa_session, account_id=self.account_id)

        rv = OrderedDict([
            ('All', [
                {'value': "[Depot].[All Depots]", 'label': 'All Depots'},
                {'value': "[Depot].[All Depots].Children", 'label': 'All Depots (Children)'},
            ])
        ])

        for depot in tnt_c.get_depots():
            if not rv.has_key(depot.country.name):
                rv[depot.country.name] = []

            rv[depot.country.name].append({'value': '[Depot].[%s]' % depot.default_code, 'label': depot.label})

        return rv


    def get_service_options(self):
        tnt_c = TntController(self.sa_session, account_id=self.account_id)

        rv = OrderedDict([
            ('All', [
                {'value': "[Service].[All Services]", 'label': 'All Services'},
                {'value': "[Service].[All Services].Children", 'label': 'All Services (Children)'},
            ])
        ])

        rv [_('Services')] = [ {'value': '[Service].[%s]' % service.label, 'label': service.label} for service in tnt_c.get_services() ]

        return rv

    def get_location_options(self):
        return {_('Locations'): [
            {'value': "[Location].[All Locations]", 'label': 'All Locations'},
            {'value': "MEMBER [Location].[Domestic] AS '[Location].[D]'", 'label': 'Domestic'},
            {'value': "MEMBER [Location].[International] AS '[Location].[I]'", 'label': 'International'},
        ]}

    def get_revenue_band_options(self):
        tnt_c = TntController(self.sa_session, account_id=self.account_id)

        rv = OrderedDict([
            ('All', [
                {'value': "[Revenue Band].[All Revenue Bands]", 'label': 'All Revenue Bands'},
                {'value': "[Revenue Band].[All Revenue Bands].Children", 'label': 'All Revenue Bands (Children)'},
            ]),
            ( _('Revenue Bands'), [ {'value': '[Revenue Band].[%s]' % rb, 'label': rb} for rb in tnt_c.get_revenue_bands()] )
        ])

        return rv


    def get_respondent_type_options(self):
        return {_('Respondent Types'): [
            {'value': "[Respondent Type].[All Respondent Types]", 'label': 'All Respondent Types'},
            {'value': "MEMBER [Respondent Type].[Internet] AS '[Respondent Type].[I]'", 'label': 'Internet'},
            {'value': "MEMBER [Respondent Type].[Paper] AS '[Respondent Type].[P]'", 'label': 'Paper'},
        ]}

    def get_general_questions_answer_options(self):
        return {_('General Questions Answers'): [
            {'value': "[Measures].[No. of Answers]", 'label': 'Respondents'},
            {'value': "[Measures].[No. of Answers %]", 'label': 'Respondents %'},
        ]}

    def get_general_questions_attributes_options(self):
            ac = TntCLMAttributeController(self.sa_session, StateManager(None, self.account_id, None))
            rv = {
                _('Single choice'): [
                    {'value': '[Attribute].[Single]', 'label': 'All single choice questions'},
                    {'value': '[Attribute].[Single].Children', 'label': 'All single choice questions (Children)'},
                ],

                #_('Multiple choice'): [{'value': '[Attribute].[Multi].[%s]' % a.default_label, 'label': a.default_label} for a in ac.get_attributes_by_type(TntCLMAttribute.TYPE_MULTIPLE_CHOICE)]
            }

            for a in ac.get_attributes_by_type(TntCLMAttribute.TYPE_SINGLE_CHOICE):

                v = '[Attribute].[Single].[%s]' % a.default_label

                # hackerdefack:
                # http://trac.office.basesoft.com/iqportal/ticket/2096
                if a.id == 32:
                    v = '[Attribute].[Scale].[%s]' % a.default_label

                rv[_('Single choice')].append({'value': v, 'label': a.default_label})

            return rv


    def get_overall_satisfaction_data(self, config, target):
        sfr = TntCLMSatisfactionReporter(self.sa_session, account_id=self.account_id)
        return sfr.get_mdx_config(config, target)

    def get_overall_satisfaction_graphs_data(self, config, target):
        sfr = TntCLMSatisfactionGraphsReporter(self.sa_session, account_id=self.account_id)
        return sfr.get_mdx_config(config, target)

    def get_action_planning_data(self, config, target):
        apr = TntCLMActionPlanningReport(self.sa_session, account_id=self.account_id)
        return apr.get_mdx_config(config, target)

    def get_ncs_league_table_data(self, config, target):
        ltr = TntCLMNCSLeagueTableReport(self.sa_session, account_id=self.account_id)
        return ltr.get_mdx_config(config, target)

    def get_loyalty_league_table_data(self, config, target):
        ltr = TntCLMLoyaltyLeagueTableReport(self.sa_session, account_id=self.account_id)
        return ltr.get_mdx_config(config, target)

    def get_loyalty_score_data(self, config, target):
        lsr = TntCLMLoyaltyScoreReporter(self.sa_session, account_id=self.account_id)
        return lsr.get_mdx_config(config, target)

    def get_general_guestions_data(self, config, target):
        r = TntCLMGeneralQuestionsReporter(self.sa_session, account_id=self.account_id)
        return r.get_mdx_config(config, target)

class SatisfactionGraphHandler(ResultHandler):
    """
    Make special graphs - use tnt chart wrapper
    """

    def __init__(self, request, **kwargs):
        ResultHandler.__init__(self, request)

class TrueReportHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.account_id = request.session['user_id']

    def get_grouped_answers_options(self, datasource, cube, config, values):
        from iqportal.iqube.core.controllers import DataSourceController, ConfigurableWidgetController
        from django.utils import simplejson
        from iqportal.tnt.clm.models import TntCLMAttributeAnswer

        dsc = DataSourceController(self.sa_session, account_id=self.account_id)
        ds = dsc.get_datasources(datasource).pop()
        cwc = ConfigurableWidgetController(self.sa_session, account_id=self.account_id)
        a = cwc.get_dimension_options(ds, cube, config, values)

        def deserialize(x):
            try:
                return simplejson.loads(x)
            except simplejson.JSONDecodeError:
                return x

        values = {}
        a_ids = []
        if a.has_key('Answer'):
            for x in a['Answer']:
                id = int(deserialize(x['value'])['member_name'])
                a_ids.append(id)
                values[id] = x

        answers = self.sa_session.query(TntCLMAttributeAnswer).filter(TntCLMAttributeAnswer.id.in_(a_ids)).all()
        ret = OrderedDict()
        # values related to 'All'
        for k,v in a.items():
            if k not in ['Answer', 'Calculated members']:
                ret.update([(k, v)])
        for answer in answers:
            l = answer.attribute.default_label
            if not ret.has_key(l):
                ret.update([(l, [])])
            # not all answers have label for an nps attribute, so we need to add their index too as label
            if answer.attribute.type == TntCLMAttribute.TYPE_NPS:
                p = answer.position - 1
                if answer.default_label == '':
                    values[answer.id]['label'] = p
                else:
                    values[answer.id]['label'] = '%s - %s' % (p, values[answer.id]['label'])

            ret[l].append(values[answer.id])

        if a.has_key('Calculated members'):
            ret.update([('Calculated members', a['Calculated members'])])

        return OrderedDict(ret)

    def get_participants_for_selection(self, selection, cube):

        from django.conf import settings
        from iqportal.iqube.core.controllers import DataSourceController

        dsc = DataSourceController(self.sa_session, account_id=self.account_id)
        ds = dsc.get_datasource_by('name', '%s.TNT-CLM' % settings.SETTINGS_MODULE).pop()

        reporter = TntCLMCellClickReporter(self.sa_session, selection, ds, cube, self.account_id)
        return reporter.get_result()

handlers = (ReportHandler, BusinessFunctionHandler, SatisfactionGraphHandler, TrueReportHandler)
