# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
This file contains the objects, functions and variables to setup the Mailing app
"""

import datetime
from sqlalchemy import Column, Integer, ForeignKey, DateTime, UnicodeText, SmallInteger, Unicode, Enum
from basesoft.sqlalchemy.table import IQTable
from basesoft.web.contrib.license import models as license_models
from django.utils.translation import ugettext as _
from sqlalchemy.schema import Index

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        aggTntCLMWaveResponse = IQTable('aggTntCLMWaveResponse', metadata,
            Column('wave_id', Integer, ForeignKey('tblTntCLMWave.id'), ),
            Column('locale_id', Integer, ForeignKey('tblTntCLMLocale.id')),
            Column('country_id', Integer, ForeignKey('tblCountry.id')),
            Column('service_id', Integer, ForeignKey('tblTntCLMService.id')),
            Column('location', Enum('D', 'I'), nullable=True, index=True),
            Column('location_2015', Enum('D', 'I'), nullable=True, index=True, default=None),
            Column('depot_id', Integer, ForeignKey('tblTntDepot.id')),
            Column('revenue_band', Unicode(5), index=True),
            Column('respondent_type', Enum('I', 'P'), nullable=True, index=True),
            Column('attribute_id', Integer, ForeignKey('tblTntCLMWaveAttribute.id')),
            Column('attribute_position', Integer),
            Column('attribute_type', UnicodeText),
            Column('answer_id', Integer, ForeignKey('tblTntCLMWaveAnswer.id')),
            Column('wave_scale_answer_id', Integer, ForeignKey('tblTntCLMWaveScaleAnswer.id')),
            Column('attribute_text', UnicodeText),
            Column('answer_text', UnicodeText),
            Column('response_int', Integer),
            Column('row_count', Integer),
            Column('created', DateTime),
            mysql_engine='MyISAM',
        )

        aggTntCLMTrueWaveResponse = IQTable('aggTntCLMTrueWaveResponse', metadata,
            Column('answer_id', Integer, index=True, nullable=True),
            Column('attribute_type', Integer, index=True, nullable=False),
            Column('attribute_id', Integer, index=True, nullable=False),
            Column('scale_answer_id', Integer, index=True, nullable=True),
            Column('country_id', Integer, index=True),
            Column('depot_id', Integer, index=True),
            Column('service_id', Integer, index=True),
            Column('is_main', Integer, index=True),
            Column('wave_id', Integer, index=True),
            Column('revenue_band', Unicode(1), index=True),
            Column('location', Unicode(1), index=True),
            Column('location_2015', Enum('D', 'I'), nullable=True, index=True, default=None),
            Column('is_respondent', Integer, index=True),
            Column('is_processed', Integer, index=True),
            Column('responses', Integer),
            Column('fact_count', Integer),
            Column('created', DateTime),
            mysql_engine='MyISAM',
        )

        aggTntCLMGeolevelHierarchy = IQTable('aggTntCLMGeolevelHierarchy', metadata,
            Column('business_unit_id', Integer, index=True),
            Column('business_unit', UnicodeText),
            Column('region_id', Integer, index=True),
            Column('region', UnicodeText),
            Column('subregion_id', Integer, index=True),
            Column('subregion', UnicodeText),
            Column('country_id', Integer, index=True),
            Column('country', UnicodeText),
            Column('created', DateTime),
            mysql_engine='MyISAM',
        )
        t = aggTntCLMGeolevelHierarchy
        Index('ix_b_r', t.c.business_unit_id, t.c.region_id)
        Index('ix_b_r_s', t.c.business_unit_id, t.c.region_id, t.c.subregion_id)
        Index('ix_b_r_s_c', t.c.business_unit_id, t.c.region_id, t.c.subregion_id, t.c.country_id)
        Index('ix_b_c', t.c.business_unit_id, t.c.country_id)
        Index('ix_r_s', t.c.region_id, t.c.subregion_id)
        Index('ix_r_s_c', t.c.region_id, t.c.subregion_id,t.c.country_id)
        Index('ix_r_c', t.c.region_id, t.c.subregion_id,t.c.country_id)
        Index('ix_s_c', t.c.subregion_id,t.c.country_id)

        aggTntCLMTntCLMWave = IQTable('aggTntCLMTntCLMWave', metadata,
            Column('wave_id', Integer, index=True),
            Column('wave_code', UnicodeText),
            Column('wave_is_main_id', Integer, index=True),
            Column('wave_is_main_label', UnicodeText),
            mysql_engine='MyISAM',
        )
        Index('ix_w_m', aggTntCLMTntCLMWave.c.wave_id, aggTntCLMTntCLMWave.c.wave_is_main_id)

        aggTntCLMFullWaveResponse = IQTable('aggTntCLMFullWaveResponse', metadata,
            Column('id', Integer, primary_key=True, index=True, nullable=False),
            Column('wave_id', Integer, index=True, nullable=False),
            Column('participant_id', Integer, index=True, nullable=False),
            Column('attribute_id', Integer, index=True, nullable=False),
            Column('answer_id', Integer, index=True, nullable=True),
            Column('scale_answer_id', Integer, index=True, nullable=True),
            Column('participant_session_id', Integer, index=True, nullable=False),
            Column('response_text', UnicodeText, nullable=True),
            Column('invitee_id', Integer, index=True, nullable=False),
            Column('depot_id', Integer, index=True, nullable=False),
            Column('locale_id', Integer, index=True, nullable=False),
            Column('service_id', Integer, index=True, nullable=False),
            Column('location', Unicode(1), index=True, nullable=False),
            Column('location_2015', Enum('D', 'I'), nullable=True, index=True, default=None),
            Column('revenue_band', Unicode(5), index=True, nullable=False),
            Column('is_respondent', Integer, index=True, nullable=False),
            Column('is_processed', Integer, index=True, nullable=False),
            Column('created', DateTime, index=True, nullable=False),
            Column('last_updated', DateTime, index=True, nullable=False),
            mysql_engine='MyISAM',
        )

        # Tnt specific dashboard widgets
        tblTntCLMDashboardWidget = IQTable('tblTntCLMDashboardWidget', metadata,
            Column('widget_id', Integer, ForeignKey('tblDashboardWidget.id', ondelete='CASCADE'), primary_key=True),

            Column('show_table', SmallInteger, nullable=False, default=1),
            Column('table_attributes', UnicodeText, nullable=False),

            Column('show_graph', SmallInteger, nullable=False, default=1),
            Column('graph_attributes', UnicodeText, nullable=False),

            Column('split_by', UnicodeText, nullable=True),
            Column('on_rows', UnicodeText, nullable=True),
            Column('rank_by', UnicodeText, nullable=True),

            Column('show_sample_size_warning', SmallInteger, nullable=False, default=1),
            Column('show_geolevel_selection', SmallInteger, nullable=False, default=1),

            Column('locked', SmallInteger, nullable=False, default=0),
            Column('decimals', SmallInteger, nullable=False, default=2),

            # json encoded dict with different widget attributes (that might not apply to all tnt widgets)
            Column('misc', UnicodeText, nullable=True),
        )

        # only grouped filters have multiple items
        tblTntCLMDashboardWidgetFilter = IQTable('tblTntCLMDashboardWidgetFilter', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('widget_id', Integer, ForeignKey('tblTntCLMDashboardWidget.widget_id', ondelete='CASCADE'),
                   nullable=False),

            Column('mdx_member_id', Integer, ForeignKey('tblMdxMember.id')),
            Column('calculated_member_id', Integer, ForeignKey('tblMdxCalculatedMember.id')),
            #constant defined in the models
            Column('category', Integer, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tables['aggTntCLMWaveResponse'] = aggTntCLMWaveResponse
        tables['aggTntCLMTrueWaveResponse'] = aggTntCLMTrueWaveResponse
        tables['aggTntCLMGeolevelHierarchy'] = aggTntCLMGeolevelHierarchy
        tables['aggTntCLMTntCLMWave'] = aggTntCLMTntCLMWave
        tables['aggTntCLMFullWaveResponse'] = aggTntCLMFullWaveResponse
        tables['tblTntCLMDashboardWidget'] = tblTntCLMDashboardWidget
        tables['tblTntCLMDashboardWidgetFilter'] = tblTntCLMDashboardWidgetFilter

        __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if not __mapping_defined:
        from sqlalchemy.orm import mapper, relation
        from iqportal.iqube.core import models as iqube_models
        from iqportal.tnt.report import models as report_models
        from basesoft.web.contrib.dashboard.models import DashboardWidget

        __mapping_defined = True

        mapper(report_models.TntCLMDashboardWidget, tables['tblTntCLMDashboardWidget'],
            properties = {
                'filters': relation(report_models.TntCLMDashboardWidgetFilter, backref='widget', cascade="all",
                                    passive_deletes=True),
            },
            polymorphic_identity='TntCLMDashboardWidget',
            inherits=DashboardWidget
        )

        mapper(report_models.TntCLMDashboardWidgetFilter, tables['tblTntCLMDashboardWidgetFilter'],
               properties = {
                    'mdx_member': relation(iqube_models.MdxMember),
                    'calculated_member': relation(iqube_models.MdxCalculatedMember),
                }
        )

def fill_date(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'TNT-CLM')

def app_permissions():
    return [
        license_models.Permission(u'tnt_view_business_function', u"""
            This gives the right to view TNT Business Functions/Clusters.
            """, _("View business functions/clusters")),
        license_models.Permission(u'tnt_create_business_function', u"""
            This gives the right to create TNT Business Functions/Clusters.
            """, _("Create business functions/clusters")),
        license_models.Permission(u'tnt_update_business_function', u"""
            This gives the right to update TNT Business Function/Clusters.
            """, _("Update business functions/clusters")),
        license_models.Permission(u'tnt_delete_business_function', u"""
            This gives the right to delete TNT Business Functions/Clusters.
            """, _("Delete business functions/clusters")),

        license_models.Permission(u'tnt_manage_global_business_function', u"""
            This gives the right to manage Global TNT Business Functions/Clusters.
            """, _("Manage global business functions/clusters")),
    ]

