# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

import os

from django.template import RequestContext
from django.shortcuts import render_to_response

from iqportal.tnt.response.controllers import ParticipantController, ResponseController
from basesoft.web.core.exceptions import PublicUserException
from basesoft.util.image import direct_image
from django.conf import settings
from basesoft.web.contrib.account.decorators import login_required
from django.http import HttpResponse
from basesoft.web.contrib.branding.controllers import LicenseThemeController

# we need to make sure that we serialize requests per session
from threading import Lock
session_locks = {}
get_set_session_lock = Lock()

def bootstrap(request, version, type, extra_requires):
    requires = ['iq_core', 'iq_loader']

    # parse extra_requires
    if extra_requires == '':
        extra_requires = []
    else:
        extra_requires = extra_requires.split(',')
    requires.extend(extra_requires)
    from basesoft.web.core.views import loader_get

    return loader_get(request, version, type, '', ','.join(requires))

def display(request, hash):
    """
    """
    sa_session = request.Session()

    user_id = None
    if request.session.has_key('user_id'):
        user_id = request.session['user_id']

    ltc = LicenseThemeController(sa_session, account_id=user_id, request=request)
    current_theme = ltc.get_current_theme()
    theme_css = current_theme.css_class
    theme_module_name = current_theme.module_name
    request.session['theme_module_name'] = theme_module_name

    # grab the global lock to ensure no race condition for creating and then acquiring the per-session lock (bug #3469)
    session_id = request.session.session_key
    with get_set_session_lock:
        if session_id not in session_locks:
            session_locks[session_id] = Lock()
        session_locks[session_id].acquire()
    try:
        pc = ParticipantController(sa_session, hash)
    except PublicUserException, e:
        # render TNT page that displays the output nicely.
        return render_to_response('login.html', {'error': e.message}, context_instance=RequestContext(request))
    finally:
        #release the per-session lock
        session_locks[session_id].release()

    return render_to_response('wave.html', {
        'invitee_hash': hash,
        'participant_session_id': pc.participant_session.id,
        'wave_type': pc.participant.invitee.wave.type.module,
        'title': pc.participant.invitee.wave.code,
        'theme_css': theme_css
    }, context_instance=RequestContext(request))

def direct_graph(request, hash):
    """
    No login is required for a direct link to a graph like this.
    This will return the image that can be used as src
    """
    return direct_image(os.path.join(settings.WRITABLE_DIRECTORY, 'hashed_graph', ("%s.png" % hash) ))

def login(request):
    """
    In case a TNT participant enters the url used for logging in, render a login page.
    """
    return render_to_response('login.html', {}, context_instance=RequestContext(request))

@login_required
def download_response_pdf(request, participant_id, locale_id=None):
    """
    returns a response with pdf data in it, which contains the filled-in responses
    """

    participant_id = int(participant_id)
    try: locale_id = int(locale_id)
    except ValueError: locale_id = None

    # create response object which will be passed into the builder to write the pdf data into
    response = HttpResponse(mimetype='application/pdf')

    # build pdf
    controller = ResponseController(request.Session(), account_id=request.session['user_id'])
    filename = controller.export_response_pdf(response, participant_id, locale_id)

    # set filename in response
    response['Content-Disposition'] = 'attachment; filename=%s' % filename

    return response
