# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _
from sqlalchemy import func, case

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryField, CustomQueryModel
from basesoft.web.contrib.i18n.models import Country
from basesoft.web.contrib.account.models import Account
from basesoft.web.contrib.contact.models import Person

from iqportal.tnt.core.models import TntDepot
from iqportal.tnt.sr.models import TntSRQualityAuditResponse

class AuditCustomQuery(CustomQuery):
    permissions = ['view_tnt_sr_audit_form']

    def tables(self):
        return (
            CustomQueryModel(TntSRQualityAuditResponse, priv=None),
            CustomQueryModel(Country, onclause=TntSRQualityAuditResponse.country_id==Country.id),
            CustomQueryModel(Account, onclause=TntSRQualityAuditResponse.account_id==Account.id),
            CustomQueryModel(Person, onclause=Account.person_id==Person.id, use_outerjoin=True, priv=None),
            CustomQueryModel(TntDepot, onclause=TntSRQualityAuditResponse.depot_id==TntDepot.id, priv=None, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(TntSRQualityAuditResponse.id, _('ID')),
            CustomQueryField(case([(Account.person_id==None, Account.name)],
                else_=func.concat(
                    case([(Person.first_name!=None, Person.first_name)], else_=""), " ",
                    case([(Person.middle_name!=None, Person.middle_name)], else_=""), " ",
                    case([(Person.last_name!=None, Person.last_name)], else_="")
                )),
                _('Account'), key="account", is_filter=True, is_group_by=True),
            CustomQueryField(TntSRQualityAuditResponse.auditee, _('Auditee'), is_filter=True, is_group_by=True),
            CustomQueryField(TntSRQualityAuditResponse.supervisor_name, _('Supervisor'), is_filter=True),
            CustomQueryField(TntSRQualityAuditResponse.date_of_audit, _('Date of audit'), is_filter=True),
            CustomQueryField(TntSRQualityAuditResponse.date_case_opened, _('Case opened'), is_filter=True),
            CustomQueryField(Country.name, _('Country'), is_group_by=True),
            CustomQueryField(Country.id, _('Country'), is_filter=True, filter_options=Country.name, in_select=False),
            CustomQueryField(TntDepot.default_code, _('Depot')),
            CustomQueryField(TntDepot.id, _('Depot'), is_filter=True, filter_options=TntDepot.default_code, in_select=False),
            CustomQueryField(TntSRQualityAuditResponse.total_score, _('Total score'), is_filter=True),
            CustomQueryField(TntSRQualityAuditResponse.contact_number, _('Contact number')),
            CustomQueryField(TntSRQualityAuditResponse.contact_name, _('Contact name')),
            CustomQueryField(TntSRQualityAuditResponse.telephone_number, _('Phone')),
            CustomQueryField(TntSRQualityAuditResponse.customer_perspective_question1, _('Were you satisfied with when you were notified of the problem (time)?'), is_filter=True, in_select=False),
            CustomQueryField(TntSRQualityAuditResponse.customer_perspective_question2, _('Were you satisfied with the explanation of the circumstances?'), is_filter=True, in_select=False),
            CustomQueryField(TntSRQualityAuditResponse.customer_perspective_question3, _('Were you satisfied that TNT kept you fully informed and updated?'), is_filter=True, in_select=False),
            CustomQueryField(TntSRQualityAuditResponse.customer_perspective_question4, _('Were you satisfied with how quickly the problem was resolved?'), is_filter=True, in_select=False),
            CustomQueryField(TntSRQualityAuditResponse.customer_perspective_question5, _('How would you rate the professionalism of the CS Representative?'), is_filter=True, in_select=False),
            CustomQueryField(TntSRQualityAuditResponse.customer_perspective_question6, _('Were you satisfied with the final solution to the problem?'), is_filter=True, in_select=False),

            CustomQueryField(TntSRQualityAuditResponse.data_capture_question2_a, _('Have the contact and telephone number fields been completed?'), is_filter=True, in_select=False),
            CustomQueryField(TntSRQualityAuditResponse.data_capture_question2_b, _('Was the case opened and closed at the correct time?'), is_filter=True, in_select=False),
            CustomQueryField(TntSRQualityAuditResponse.data_capture_question3_a, _('Do the remarks explain the issue?'), is_filter=True, in_select=False),
            CustomQueryField(TntSRQualityAuditResponse.data_capture_question3_b, _('Has the correct Complaint type been used?'), is_filter=True, in_select=False),
            CustomQueryField(TntSRQualityAuditResponse.data_capture_question3_c, _('Has the correct Cause Type been used?'), is_filter=True, in_select=False),
            CustomQueryField(TntSRQualityAuditResponse.data_capture_question3_d, _('Have the correct Action types with costs (if applicable) been entered?'), is_filter=True, in_select=False),

            CustomQueryField(TntSRQualityAuditResponse.supervisor_perspective_question2, _('Has the training been given?'), is_filter=True, is_group_by=True, in_select=False, filter_options=TntSRQualityAuditResponse.supervisor_perspective_question2),
)

custom_queries = (AuditCustomQuery, )
