# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon

class QualityAuditFormIcon(IconBarIcon):
    """
    Opens the interface for product overview manager
    """

    id = 'icon-sr-quality-audit-form'
    title = _('Quality Audit Form')
    icon_index = 81
    component = 'tnt_sr_quality_audit_panel'
    group = 'tnt-sr'
    colorname = 'iconbar-spindle'

    char_main = u'\uE800' # World icon
    char_sub = u'\uE81B' # Tablet icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('create_tnt_sr_audit_form')

class QualityAuditManagerIcon(IconBarIcon):
    """
    Opens the interface for product overview manager
    """

    id = 'icon-sr-quality-audit-manager'
    title = _('Quality Audit listing')
    icon_index = 81
    component = 'tnt_sr_quality_audit_manager'
    group = 'tnt-sr'
    colorname = 'iconbar-spindle'

    char_main = u'\uE800' # World icon
    char_sub = u'\uE821' # Notebook icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('view_tnt_sr_audit_form')

def get_icons():
    """
    return the list of icons that the tnt-sr app adds to the iconbar
    """
    return []
